# Part 2 
# based on of_tutorial by James McCauley

from pox.core import core # core functionality for the POX controller
import pox.openflow.libopenflow_01 as of #  Defines the OpenFlow 1.0 protocol messages

log = core.getLogger()


class Firewall(object):
    """
    A Firewall object is created for each switch that connects.
    A Connection object for that switch is passed to the __init__ function.
    """

    def __init__(self, connection):
        # Keep track of the connection to the switch so that we can
        # send it messages!
        self.connection = connection

        # This binds our PacketIn event listener (adds listeners to the connection object, allowing the controller to react to events from the switch, such as packet-in messages)
        connection.addListeners(self)

        # add switch rules here
        # allow ARP traffic (Match packets with the Ethernet type 0x0806, Apply the action of flooding the packet out all ports of the switch, Priority 3)
        self.connection.send(of.ofp_flow_mod(action = of.ofp_action_output( port =  of.OFPP_FLOOD),
        	priority=3,
        	match=of.ofp_match( dl_type = 0x0806 ))) # sends an OpenFlow flow modification message to the switch, of.ofp_flow_mod: is used to install a new flow entry in the switch's flow table
        # allow ICMP traffic, flooding to all ports
        self.connection.send(of.ofp_flow_mod(action = of.ofp_action_output( port =  of.OFPP_FLOOD),
        	priority=3,
        	match=of.ofp_match( dl_type = 0x0800, nw_proto = 1))) # l_type=0x0800 (Ethernet type for IPv4), nw_proto=1 (IP protocol number for ICMP)

    def _handle_PacketIn(self, event):
        """
        Packets not handled by the router rules will be
        forwarded to this method to be handled by the controller
        """

        packet = event.parsed  # This is the parsed packet data.
        if not packet.parsed:
            log.warning("Ignoring incomplete packet")
            return

        packet_in = event.ofp  # The actual ofp_packet_in message.
        print("Unhandled packet :" + str(packet.dump()))


def launch():
    """
    Starts the component
    """

    def start_switch(event):
        log.debug("Controlling %s" % (event.connection,))
        Firewall(event.connection)

    core.openflow.addListenerByName("ConnectionUp", start_switch)
